@@ -6,7 +6,7 @@
 #   Filter converting cyrillic text into transliterated one.
 #
 # AUTHOR
-#   Igor Lobanov   <igor.lobanov@gmail.com>
+#   Igor Lobanov <igor.lobanov@gmail.com>
 #
 # COPYRIGHT
 #   Copyright (C) 2004 Igor Lobanov.  All Rights Reserved.
@@ -22,7 +22,9 @@ use vars qw( $VERSION );
 use Template::Plugin;
 use base qw( Template::Plugin );
 
-$VERSION = sprintf("%d.%02d", q$Revision: 0.04 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 0.05 $ =~ /(\d+)\.(\d+)/);
+
+my $DEFAULT_CHARSET = 'koi';
 
 # (en|de)code table
 my $tab = {
@@ -30,37 +32,37 @@ my $tab = {
 		# {1} => {1}
 		single	=> [
 			'',
-			"abvgdezijklmnoprstuf'\"ABVGDEZIJKLMNOPRSTUF'\""
+			"abvgdezijklmnoprstuf'y\"ABVGDEZIJKLMNOPRSTUF'Y\""
 		],
 		# +	=> {2,}
 		plural	=> [
 			# 0: re with plural-transliterated letters and special cases
-			'([ţ]||||||||||||[]|||||||||||)',
+			'(|[ţ]|[ţ]||||||||||||[]|[]||||||||||)',
 			# 1: table for these letters and cases [0]
 			{
-				''		=> 'yo',
-				''		=> 'Yo',
-				''		=> 'zh',
-				''		=> 'Zh',
-				''		=> 'kh',
-				''		=> 'Kh',
-				''		=> 'tc',
-				''		=> 'Tc',
-				''		=> 'ch',
-				''		=> 'Ch',
-				''		=> 'sh',
-				''		=> 'Sh',
-				''		=> 'shch',
-				''		=> 'Shch',
-				''		=> 'yi',
-				''		=> 'Yi',
-				''		=> 'ye',
-				''		=> 'Ye',
-				''		=> 'yu',
-				''		=> 'Yu',
-				''		=> 'ya',
-				''		=> 'Ya',
-				# {2} => {2,3} - could not be at the begining of the word
+				''	=> 'yo',
+				''	=> 'Yo',
+				''	=> 'zh',
+				''	=> 'Zh',
+				''	=> 'kh',
+				''	=> 'Kh',
+				''	=> 'tc',
+				''	=> 'Tc',
+				''	=> 'ch',
+				''	=> 'Ch',
+				''	=> 'sh',
+				''	=> 'Sh',
+				''	=> 'shch',
+				''	=> 'Shch',
+				''	=> 'y',
+				''	=> 'Y',
+				''	=> 'ye',
+				''	=> 'Ye',
+				''	=> 'yu',
+				''	=> 'Yu',
+				''	=> 'ya',
+				''	=> 'Ya',
+				# {2} => {3} - could not be at the begining of the word
 				''	=> 'jie',
 				''	=> 'JIE',
 				'أ'	=> 'jio',
@@ -69,9 +71,22 @@ my $tab = {
 				''	=> 'JIU',
 				''	=> 'jia',
 				''	=> 'JIA',
+				''	=> 'yje',
+				''	=> 'YJE',
+				'٣'	=> 'yjo',
+				''	=> 'YJO',
+				''	=> 'yju',
+				''	=> 'YJU',
+				''	=> 'yja',
+				''	=> 'YJA',
+				''	=> 'yiu',
+				''	=> 'YIU',
+				# {3} => {3,5} - could not be at the begining of the word
+				''	=> 'uisch',
+				''	=> 'UISCH',
 			},
 			# 3: re with special transliterated escapes
-			'(shch|Shch|SHCH|tch|TCH|Tch|ji[eoua]|JI[EOUA]|yo|zh|kh|tc|ch|sh|yi|ye|yu|ya|Y[Oo]|Z[Hh]|K[Hh]|T[Cc]|C[Hh]|S[Hh]|Y[Ii]|Y[Ee]|Y[Uu]|Y[Aa])',
+			'(uisch|UISCH|shch|Shch|SHCH|tch|TCH|Tch|ji[eoua]|JI[EOUA]|yj[eoua]|yiu|YIU|YJ[EOUA]|yo|zh|kh|tc|ch|sh|ye|yu|ya|Y[Oo]|Z[Hh]|K[Hh]|T[Cc]|C[Hh]|S[Hh]|Y[Ee]|Y[Uu]|Y[Aa])',
 			# 4: table for special transliterated escapes [3]
 			{
 				'shch'	=> '',
@@ -88,13 +103,22 @@ my $tab = {
 				'JIO'	=> '',
 				'JIU'	=> '',
 				'JIA'	=> '',
+				'yje'	=> '',
+				'yjo'	=> '٣',
+				'yju'	=> '',
+				'yja'	=> '',
+				'yiu'	=> '',
+				'YJE'	=> '',
+				'YJO'	=> '',
+				'YJU'	=> '',
+				'YJA'	=> '',
+				'YIU'	=> '',
 				'yo'	=> '',
 				'zh'	=> '',
 				'kh'	=> '',
 				'tc'	=> '',
 				'ch'	=> '',
 				'sh'	=> '',
-				'yi'	=> '',
 				'ye'	=> '',
 				'yu'	=> '',
 				'ya'	=> '',
@@ -110,14 +134,14 @@ my $tab = {
 				'Ch'	=> '',
 				'SH'	=> '',
 				'Sh'	=> '',
-				'YI'	=> '',
-				'Yi'	=> '',
 				'YE'	=> '',
 				'Ye'	=> '',
 				'YU'	=> '',
 				'Yu'	=> '',
 				'YA'	=> '',
 				'Ya'	=> '',
+				'uisch'	=> '',
+				'UISCH'	=> '',
 			}
 		]
 	},
@@ -125,37 +149,37 @@ my $tab = {
 		# {1} => {1}
 		single	=> [
 			'',
-			"abvgdezijklmnoprstuf'\"ABVGDEZIJKLMNOPRSTUF'\""
+			"abvgdezijklmnoprstuf'y\"ABVGDEZIJKLMNOPRSTUF'Y\""
 		],
 		# +	=> {2,}
 		plural	=> [
 			# 0: re with plural-transliterated letters and special cases
-			'([]||||||||||||[Ũ]|||||||||||)',
+			'(|[]|[]||||||||||||[Ũ]|[Ũ]||||||||||)',
 			# 1: table for these letters and cases [0]
 			{
-				''		=> 'yo',
-				''		=> 'Yo',
-				''		=> 'zh',
-				''		=> 'Zh',
-				''		=> 'kh',
-				''		=> 'Kh',
-				''		=> 'tc',
-				''		=> 'Tc',
-				''		=> 'ch',
-				''		=> 'Ch',
-				''		=> 'sh',
-				''		=> 'Sh',
-				''		=> 'shch',
-				''		=> 'Shch',
-				''		=> 'yi',
-				''		=> 'Yi',
-				''		=> 'ye',
-				''		=> 'Ye',
-				''		=> 'yu',
-				''		=> 'Yu',
-				''		=> 'ya',
-				''		=> 'Ya',
-				# {2} => {2,3} - could not be at the begining of the word
+				''	=> 'yo',
+				''	=> 'Yo',
+				''	=> 'zh',
+				''	=> 'Zh',
+				''	=> 'kh',
+				''	=> 'Kh',
+				''	=> 'tc',
+				''	=> 'Tc',
+				''	=> 'ch',
+				''	=> 'Ch',
+				''	=> 'sh',
+				''	=> 'Sh',
+				''	=> 'shch',
+				''	=> 'Shch',
+				''	=> 'y',
+				''	=> 'Y',
+				''	=> 'ye',
+				''	=> 'Ye',
+				''	=> 'yu',
+				''	=> 'Yu',
+				''	=> 'ya',
+				''	=> 'Ya',
+				# {2} => {3} - could not be at the begining of the word
 				''	=> 'jie',
 				''	=> 'JIE',
 				''	=> 'jio',
@@ -164,9 +188,22 @@ my $tab = {
 				''	=> 'JIU',
 				''	=> 'jia',
 				''	=> 'JIA',
+				''	=> 'yje',
+				''	=> 'YJE',
+				''	=> 'yjo',
+				'ۨ'	=> 'YJO',
+				''	=> 'yju',
+				''	=> 'YJU',
+				''	=> 'yja',
+				''	=> 'YJA',
+				''	=> 'yiu',
+				''	=> 'YIU',
+				# {3} => {3,5} - could not be at the begining of the word
+				''	=> 'uisch',
+				''	=> 'UISCH',
 			},
 			# 3: re with special transliterated escapes
-			'(shch|Shch|SHCH|tch|TCH|Tch|ji[eoua]|JI[EOUA]|yo|zh|kh|tc|ch|sh|yi|ye|yu|ya|Y[Oo]|Z[Hh]|K[Hh]|T[Cc]|C[Hh]|S[Hh]|Y[Ii]|Y[Ee]|Y[Uu]|Y[Aa])',
+			'(uisch|UISCH|shch|Shch|SHCH|tch|TCH|Tch|ji[eoua]|JI[EOUA]|yj[eoua]|yiu|YIU|YJ[EOUA]|yo|zh|kh|tc|ch|sh|ye|yu|ya|Y[Oo]|Z[Hh]|K[Hh]|T[Cc]|C[Hh]|S[Hh]|Y[Ee]|Y[Uu]|Y[Aa])',
 			# 4: table for special transliterated escapes [3]
 			{
 				'shch'	=> '',
@@ -183,13 +220,22 @@ my $tab = {
 				'JIO'	=> 'ܨ',
 				'JIU'	=> '',
 				'JIA'	=> '',
+				'yje'	=> '',
+				'yjo'	=> '',
+				'yju'	=> '',
+				'yja'	=> '',
+				'yiu'	=> '',
+				'YJE'	=> '',
+				'YJO'	=> 'ۨ',
+				'YJU'	=> '',
+				'YJA'	=> '',
+				'YIU'	=> '',
 				'yo'	=> '',
 				'zh'	=> '',
 				'kh'	=> '',
 				'tc'	=> '',
 				'ch'	=> '',
 				'sh'	=> '',
-				'yi'	=> '',
 				'ye'	=> '',
 				'yu'	=> '',
 				'ya'	=> '',
@@ -205,14 +251,14 @@ my $tab = {
 				'Ch'	=> '',
 				'SH'	=> '',
 				'Sh'	=> '',
-				'YI'	=> '',
-				'Yi'	=> '',
 				'YE'	=> '',
 				'Ye'	=> '',
 				'YU'	=> '',
 				'Yu'	=> '',
 				'YA'	=> '',
 				'Ya'	=> '',
+				'uisch'	=> '',
+				'UISCH'	=> '',
 			}
 		]
 	}
@@ -222,8 +268,6 @@ my $tab = {
 $tab->{'windows-1251'} = $tab->{'cp1251'} = $tab->{'win'};
 $tab->{'koi8-r'} = $tab->{'koi8r'} = $tab->{'koi8'} = $tab->{'koi'};
 
-my $DEFAULT_CHARSET = 'koi';
-
 sub new {
 	my ( $class, $context, @params ) = @_;
 
@@ -343,8 +387,8 @@ source cyrillic word and result of
 B<cyrillic>->B<translit>->B<cyrillic> conversion
 (B<CTC>-conversion). Although one of the aims of this module
 is to find such correspondence, this is difficult without
-making transliterated text bad understandable. Currently 2
-main problems are known:
+making transliterated text bad understandable. Currently 1
+main problem is known:
 
 =over
 
@@ -353,27 +397,15 @@ main problems are known:
 place if you make B<CTC>-conversion with UPPER case words.
 Fortunately there are no words which starts with signs.
 
-=item * Wrong B<CTC>-conversion in rare cases where cyrillic
-letters 'B<sh>' and 'B<ch>' meets one after another. In this
-case they are converted in cyrillic letter 'B<shch>'. This
-is rare case (I met it only twice after scaning dictionary
-with 130000 words) and very rare words.
-
 =back
 
-=head1 IMPORTANT NOTE
-
-This is alpha release of module. Charset tables and other
-stuff could be changed in future versions. Use this module
-on your own risk.
-
 =head1 SEE ALSO
 
 L<Template|Template>
 
 =head1 AUTHOR
 
-Igor Lobanov, E<lt>igor.lobanov@gmail.comE<gt>
+Igor Lobanov, E<lt>liol@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
